;********************************************************************\
;
;  Name:         mscbutil.asm
;  Created by:   Stefan Ritt
;
;  Contents:     Various utility functions for MSCB protocol
;
;  $Log: mscbutil.asm,v $
;  Revision 1.1  2001/07/20 07:30:47  midas
;  Initial revision
;
;
;********************************************************************/

;--------------------------------------------------------------------
;
; CRC-8 code calculation, adds value in A to running CRC code 
; in CRC_CODE
;
; Modifies: DPTR,CRC_CODE
;
;--------------------------------------------------------------------

CRC8_DATA:       
        db 000h, 05eh, 0bch, 0e2h, 061h, 03fh, 0ddh, 083h
        db 0c2h, 09ch, 07eh, 020h, 0a3h, 0fdh, 01fh, 041h
        db 09dh, 0c3h, 021h, 07fh, 0fch, 0a2h, 040h, 01eh
        db 05fh, 001h, 0e3h, 0bdh, 03eh, 060h, 082h, 0dch
        db 023h, 07dh, 09fh, 0c1h, 042h, 01ch, 0feh, 0a0h
        db 0e1h, 0bfh, 05dh, 003h, 080h, 0deh, 03ch, 062h
        db 0beh, 0e0h, 002h, 05ch, 0dfh, 081h, 063h, 03dh
        db 07ch, 022h, 0c0h, 09eh, 01dh, 043h, 0a1h, 0ffh
        db 046h, 018h, 0fah, 0a4h, 027h, 079h, 09bh, 0c5h
        db 084h, 0dah, 038h, 066h, 0e5h, 0bbh, 059h, 007h
        db 0dbh, 085h, 067h, 039h, 0bah, 0e4h, 006h, 058h
        db 019h, 047h, 0a5h, 0fbh, 078h, 026h, 0c4h, 09ah
        db 065h, 03bh, 0d9h, 087h, 004h, 05ah, 0b8h, 0e6h
        db 0a7h, 0f9h, 01bh, 045h, 0c6h, 098h, 07ah, 024h
        db 0f8h, 0a6h, 044h, 01ah, 099h, 0c7h, 025h, 07bh
        db 03ah, 064h, 086h, 0d8h, 05bh, 005h, 0e7h, 0b9h
        db 08ch, 0d2h, 030h, 06eh, 0edh, 0b3h, 051h, 00fh
        db 04eh, 010h, 0f2h, 0ach, 02fh, 071h, 093h, 0cdh
        db 011h, 04fh, 0adh, 0f3h, 070h, 02eh, 0cch, 092h
        db 0d3h, 08dh, 06fh, 031h, 0b2h, 0ech, 00eh, 050h
        db 0afh, 0f1h, 013h, 04dh, 0ceh, 090h, 072h, 02ch
        db 06dh, 033h, 0d1h, 08fh, 00ch, 052h, 0b0h, 0eeh
        db 032h, 06ch, 08eh, 0d0h, 053h, 00dh, 0efh, 0b1h
        db 0f0h, 0aeh, 04ch, 012h, 091h, 0cfh, 02dh, 073h
        db 0cah, 094h, 076h, 028h, 0abh, 0f5h, 017h, 049h
        db 008h, 056h, 0b4h, 0eah, 069h, 037h, 0d5h, 08bh
        db 057h, 009h, 0ebh, 0b5h, 036h, 068h, 08ah, 0d4h
        db 095h, 0cbh, 029h, 077h, 0f4h, 0aah, 048h, 016h
        db 0e9h, 0b7h, 055h, 00bh, 088h, 0d6h, 034h, 06ah
        db 02bh, 075h, 097h, 0c9h, 04ah, 014h, 0f6h, 0a8h
        db 074h, 02ah, 0c8h, 096h, 015h, 04bh, 0a9h, 0f7h
        db 0b6h, 0e8h, 00ah, 054h, 0d7h, 089h, 06bh, 035h

CRC8_ADD:
        PUSH    ACC

        MOV     DPTR,#CRC8_DATA         ; Point To Table
        XRL     A,CRC_CODE              ; XOR In CRC
        MOVC    A,@A+DPTR               ; Get New CRC Byte
        MOV     CRC_CODE,A              ; Store Back

        POP     ACC
        RET

CRC8_ADD1:
        PUSH    ACC
        PUSH    DPL
        PUSH    DPH

        MOV     DPTR,#CRC8_DATA         ; Point To Table
        XRL     A,CRC_CODE              ; XOR In CRC
        MOVC    A,@A+DPTR               ; Get New CRC Byte
        MOV     CRC_CODE,A              ; Store Back

        POP     DPH
        POP     DPL
        POP     ACC
        RET

;--------------------------------------------------------------------
;
; Configure UART via timer 2 
;
; parameters:
;   A=1:9600,2:19200,3:28800,4:57600,5:115200,6:172800,7:345600 Baud
;
; Modifes: A,DPTR
;
;--------------------------------------------------------------------

BAUD_TABLE:     ; reload 100-x, x = 11.0592MHz/32/baud
        DB      100h-36,100h-18,100h-12,100h-6,100h-3,100h-2,100h-1

UART_INIT:
        MOV     SCON,#11010000b ; Mode 3, 9 bit, receive enable
;        MOV     SCON,#50H       ; Mode 1, 8 bit, receive enable

        MOV     TMOD,#00H       ; timer 0&1 no function
        MOV     T2CON,#00110100b; timer 2 RX+TX mode
        MOV     RCAP2H,#0FFh
        MOV     RCAP2L,#100h-36 ; 9600 baud by default
        SETB    TR2

        RET

SET_BAUD:
        CLR     TR2
        MOV     RCAP2H,#0FFh

        MOV     DPTR,#BAUD_TABLE
        DEC     A
        MOVC    A,@A+DPTR
        MOV     RCAP2L,A        ; set reload register from table
        SETB    TR2

        RET

;--------------------------------------------------------------------
;
; Refresh watchdog
;
; Modifes:
;
;--------------------------------------------------------------------

WATCHDOG_REFRESH:

IF(CPU_ADUC812)
        SETB    WDR1            ; refresh watchdog timer..
        SETB    WDR2            ; ..bits must be set in this order
ENDIF

IF(CPU_C8051F000)
        MOV     WDTCN,#0A5h     ; refresh watchdog timer..
ENDIF
        RET        

;--------------------------------------------------------------------
;
; Delay functions for 11.0520 MHz quartz
;
; Modifies: R6,R7,(R5)
;
;--------------------------------------------------------------------

IF(CPU_ADUC812)
DELAY_100M:                     ; delay 100ms
        MOV     R7,#200         ; 200 * 500us = 100ms
DLY1:   MOV     R6,#229         ; 229 * 2.17us = 500us
        DJNZ    R6,$            ; sit here for 500us
        DJNZ    R7,DLY1         ; repeat 200 times (100ms delay)
        RET

DELAY_10M:                      ; delay 10ms
        MOV     R7,#20          ; 20  * 500us = 10ms
DLY2:   MOV     R6,#229         ; 229 * 2.17us = 500us
        DJNZ    R6,$            ; sit here for 500us
        DJNZ    R7,DLY2         ; repeat 200 times (100ms delay)
        RET

DELAY_1M:                       ; delay 1ms
        MOV     R7,#2           ; 2  * 500us = 1ms
DLY3:   MOV     R6,#229         ; 229 * 2.17us = 500us
        DJNZ    R6,$            ; sit here for 500us
        DJNZ    R7,DLY3         ; repeat 200 times (100ms delay)
        RET

DELAY_100U:                     ; delay 100us
        MOV     R6,#43          ; 1.085us
        DJNZ    R6,$            ; 43 * 2.17us = 93.31us
        NOP                     ; 1.085us
        RET                     ; 4*1.085us (call+ret)

DELAY_10U:                      ; delay 10us
        MOV     R6,#2           ; 1.085us
        DJNZ    R6,$            ; 2 * 2.17us = 4.34us
        RET                     ; 4*1.085us (call+ret)

DELAY_1U:
        RET                     ; anyhow too long for ADuC
ENDIF

IF(CPU_C8051F000)
        ; 11059200Hz -> 1 cycle = 0.09 us

DELAY_100M:                     ; delay 100ms
        MOV     R5,#10
DLY1:   CALL    DELAY_10M
        DJNZ    R5,DLY1
        RET

DELAY_10M:                      ; delay 1ms
        MOV     R7,#200         ; 200 * 50us = 10ms
DLY2:   MOV     R6,#183         ; 0.18us
        DJNZ    R6,$            ; 183 * 0.27us = 49.41us
        DJNZ    R7,DLY2
        RET                     ; 9 * 0.09us = 0.81us (call+ret)

DELAY_1M:                       ; delay 1ms
        MOV     R7,#20          ; 20 * 50us = 1ms
DLY3:   MOV     R6,#183         ; 0.18us
        DJNZ    R6,$            ; 183 * 0.27us = 49.41us
        DJNZ    R7,DLY3
        RET                     ; 9 * 0.09us = 0.81us (call+ret)

DELAY_100U:                     ; delay 100us
        MOV     R7,#2           ; 2 * 50us
DLY4:   MOV     R6,#181         ; 0.18us
        DJNZ    R6,$            ; 181 * 0.27us = 48.87us
        DJNZ    R7,DLY4
        RET                     ; 9 * 0.09us = 0.81us (call+ret)

DELAY_10U:                      ; delay 10us
        MOV     R6,#33          ; 0.18us
        DJNZ    R6,$            ; 33 * 0.27us = 8.91us
        RET                     ; 9 * 0.09us = 0.81us (call+ret)

DELAY_1U:                       ; delay 1us
        RET                     ; 9 * 0.09us = 0.81us (call+ret)

ENDIF

;--------------------------------------------------------------------
;
; EEPROM functions for reading writing flash EEPROM
;
; Parameters: 
;   A   parameter index 0..N
;   R0  pointer to data
;   R1  data length (1..4)
;
; Modifies:
;   A,R0,R1,R2
;
;--------------------------------------------------------------------

IF(CPU_ADUC812)

EEPROM_WRITE:
        MOV     EADRL,A         ; select page
        MOV     ECON,#5         ; erase page
        MOV     EDATA1,@R0
        INC     R0
        MOV     EDATA2,@R0
        INC     R0
        MOV     EDATA3,@R0
        INC     R0
        MOV     EDATA4,@R0
        INC     R0
        MOV     ECON,#2         ; write page
        RET

EEPROM_READ:
        MOV     EADRL,A         ; set page
        MOV     ECON,#1         ; read page
        MOV     A,R1
        DEC     A
        MOV     @R0,EDATA1
        JZ      EEPROM_RET
        INC     R0
        DEC     A
        MOV     @R0,EDATA2
        JZ      EEPROM_RET
        INC     R0
        DEC     A
        MOV     @R0,EDATA3
        JZ      EEPROM_RET
        INC     R0
        DEC     A
        MOV     @R0,EDATA4
EEPROM_RET:
        RET

ENDIF

IF(CPU_C8051F000)

EEPROM_WRITE:
        ; copy page to RAM at 128
        PUSH    DPL
        PUSH    DPH
        PUSH    B
        PUSH    ACC
        MOV     B,R0
        MOV     R0,#80h
        MOV     DPTR,#7F80h
        MOV     R1,#80h
EEW_LOOP1:
        MOV     A,R1
        MOVC    A,@A+DPTR
        MOV     @R1,A
        INC     R1
        DJNZ    R0,EEW_LOOP1

        ; copy data into RAM page
        POP     ACC
        MOV     R0,B

        RL      A
        RL      A
        ADD     A,#80h
        MOV     R1,A
        MOV     R2,#4
EEW_LOOP2:
        MOV     A,@R0
        MOV     @R1,A
        INC     R1
        INC     R0
        DJNZ    R2,EEW_LOOP2

        ; erase page
        MOV     A,FLSCL
        ANL     A,#0F0h
        ORL     A,#00001000b    ; set timer for 11.052 MHz clock
        MOV     FLSCL,A
        MOV     PSCTL,#3        ; allow write+erase
        MOV     DPTR,#8000h
        MOVX    @DPTR,A

        ; write page
        MOV     PSCTL,#1        ; allow write

        MOV     R0,#80h
        MOV     DPTR,#8000h
        MOV     R1,#80h
EEW_LOOP3:
        MOV     A,@R1
        MOVX    @DPTR,A
        INC     R1
        INC     DPL
        DJNZ    R0,EEW_LOOP3

        MOV     PSCTL,#0        ; don't allow write+erase

        POP     B
        POP     DPH
        POP     DPL
        RET

EEPROM_READ:
        MOV     DPTR,#8000h     ; read from 128-byte EEPROM page
        RL      A
        RL      A
EER_LOOP:
        PUSH    ACC
        MOVC    A,@A+DPTR
        MOV     @R0,A
        POP     ACC
        INC     R0
        INC     A
        DJNZ    R1,EER_LOOP
        RET

ENDIF

;--------------------------------------------------------------------
;
; LCD functions for HD44780/KS0066 compatible LCD displays
;
; Modifes: R6,R7 (via DELAY), A, C, DPTR
;
;--------------------------------------------------------------------

LCD     EQU     P0              ; LCD display connected to port0

LCD_RS  EQU     LCD.1
LCD_R_W EQU     LCD.2
LCD_E   EQU     LCD.3

LCD_DB4 EQU     LCD.4
LCD_DB5 EQU     LCD.5
LCD_DB6 EQU     LCD.6
LCD_DB7 EQU     LCD.7

LCD_CLK EQU     LCD.7           ; pins for 4021 shift register
LCD_P_W EQU     LCD.6
LCD_SD  EQU     LCD.0

LCD_SETUP:
        MOV     LCD,#0
        CALL    DELAY_100M
        MOV     LCD,#00110000b
        SETB    LCD_E
        CALL    DELAY_1U
        CLR     LCD_E
        CALL    DELAY_100M
        SETB    LCD_E
        CALL    DELAY_1U
        CLR     LCD_E
        CALL    DELAY_100M
        SETB    LCD_E
        CALL    DELAY_1U
        CLR     LCD_E
        CALL    DELAY_100M

        MOV     LCD,#00100000b  ; set 4-bit interface
        SETB    LCD_E
        CALL    DELAY_1U
        CLR     LCD_E
        CALL    DELAY_100M

        ; test if LCD present
        MOV     LCD,#11111100b  ; data input R/W=H
        CALL    DELAY_10U
        MOV     A,LCD
        JNB     ACC.7,LCD_OK

        CLR     LCD_PRESENT
        RET

LCD_OK:
        SETB    LCD_PRESENT

        CLR     C               ; indicate control
        MOV     A,#00101100b    ; select 2 lines, big font
        CALL    LCD_OUT

        MOV     A,#00001100b    ; display on
        CALL    LCD_OUT

        MOV     A,#00000001b    ; clear display
        CALL    LCD_OUT

        MOV     A,#00000110b    ; set entry mode
        CALL    LCD_OUT

        RET

LCD_CLEAR:                      ; clear display
        PUSH    ACC
        CLR     C               ; indicate command
        MOV     A,#1
        CALL    LCD_OUT
        POP     ACC
        RET

LCD_GOTO:                       ; goto position A(0..19),B(0..1)
        PUSH    PSW
        PUSH    ACC
        MOV     C,B.0
        MOV     ACC.6,C
        SETB    ACC.7
        CLR     C
        CALL    LCD_OUT
        POP     ACC
        POP     PSW
        RET

LCD_PUTS:                       ; output zero terminated string
        PUSH    PSW
        PUSH    ACC

        SETB    C               ; indicate data
        CLR     A
        MOV     R0,A
PUTS_LOOP:
        MOV     A,R0
        MOVC    A,@A+DPTR
        JZ      PUTS_RET
        CALL    LCD_OUT
        INC     R0
        SJMP    PUTS_LOOP
PUTS_RET:
        CLR     C

        POP     ACC
        POP     PSW
        RET

LCD_HEX:                        ; output ACC in hex format
        PUSH    ACC
        PUSH    ACC
        SWAP    A
        ANL     A,#00001111b
        CALL    LCD_HEX_NIBBLE
        POP     ACC
        ANL     A,#00001111b
        CALL    LCD_HEX_NIBBLE
        POP     ACC
        RET

LCD_HEX_NIBBLE:
        CLR     C
        SUBB    A,#10
        JNC     HEX_A_F
        ADD     A,#10
        ADD     A,#'0'
        SETB    C
        CALL    LCD_OUT
        RET
HEX_A_F:
        ADD     A,#'A'
        SETB    C
        CALL    LCD_OUT
        RET       

LCD_PUTC:
        SETB    C

LCD_OUT:                        ; move ACC to LCD via 4-bit interface
        JB      LCD_PRESENT, LCD_OUT1
        RET

LCD_OUT1:
        PUSH    ACC
        PUSH    B
        MOV     B,A
        MOV     LCD,#11111100b  ; data input R/W=H
        CALL    DELAY_1U
LCD_BUSY:
        MOV     A,LCD
        JB      ACC.7,LCD_BUSY  ; loop if BF
        MOV     LCD,#0
        CALL    DELAY_1U

        MOV     A,B
        ANL     A,#11110000b
        MOV     ACC.1,C         ; set RS according to C
        MOV     LCD,A
        CALL    DELAY_1U        ; let signals settle
        SETB    LCD_E
        CALL    DELAY_1U
        CLR     LCD_E
        MOV     A,B
        SWAP    A
        ANL     A,#11110000b
        MOV     ACC.1,C
        MOV     LCD,A
        CALL    DELAY_1U        ; let signals settle
        SETB    LCD_E
        CALL    DELAY_1U
        CLR     LCD_E
        POP     B
        POP     ACC
        RET

;--------------------------------------------------------------------
;
; Input function for SCS-LCD1
;
; Modifes: R0,C, R6,R7 (via delay)
; Returns: switch states in ACC
;
;--------------------------------------------------------------------

SCS_LCD1_READ:
        CLR     LCD_CLK
        SETB    LCD_SD          ; enable input
        
        SETB    LCD_P_W         ; latch input
        CALL    DELAY_1U
        CLR     LCD_P_W
        
        MOV     C,LCD_SD        ; get first bit
        RRC     A               ; rol carry bit in A

        MOV     R0,#7
SCS_LCD1_LOOP:

        SETB    LCD_CLK
        CALL    DELAY_1U
        CLR     LCD_CLK
        CALL    DELAY_1U
        
        MOV     C,LCD_SD
        RRC     A               ; rol carry bit in A

        DJNZ    R0,SCS_LCD1_LOOP
        
        RET
