;********************************************************************\
;
;  Name:         scs-sm1.asm
;  Created by:   Stefan Ritt
;
;  Contents:     MSCB program for Cygnal sub-master
;
;  $Log: scs_sm1.asm,v $
;  Revision 1.1  2001/09/03 09:13:29  midas
;  Renamed file ("-" gave problems with A51 assembler)
;
;  Revision 1.1  2001/07/20 07:30:47  midas
;  Initial revision
;
;
;********************************************************************/

$include(C8051F200.inc)

; port mappings
LED_RED         EQU     P3.7            ; LEDs on front panel
LED_YELLOW      EQU     P0.6
LED_GREEN       EQU     P0.5

RS485_EN        EQU     P3.5            ; enable for MAX1483 transmitter


LPT_DATA        EQU     P1              ; 8 data bits to LPT

LPT_NSTROBE     EQU     P0.7            ; control/status bits to LPT
LPT_BUSY        EQU     P3.2
LPT_NDATAREADY  EQU     P3.3
LPT_NACK        EQU     P0.3
LPT_RESET       EQU     P3.0
LPT_STATUS      EQU     P3.1
LPT_BIT9        EQU     P0.4

; MSCB commands
VERSION         EQU     010h            ; protocol version 1.0

CMD_ADDR_NODE8  EQU     009h
CMD_ADDR_NODE16 EQU     00Ah
CMD_ADDR_BC     EQU     010h
CMD_ADDR_GRP8   EQU     011h
CMD_ADDR_GRP16  EQU     012h
CMD_PING8       EQU     019h
CMD_PING16      EQU     01Ah

CMD_INIT        EQU     020h
CMD_GET_INFO    EQU     028h
CMD_SET_ADDR    EQU     034h
CMD_SET_BAUD    EQU     039h

CMD_FREEZE      EQU     041h
CMD_SYNC        EQU     049h
CMD_TRANSP      EQU     050h
CMD_USER        EQU     058h

CMD_TOKEN       EQU     060h

CMD_ACK         EQU     078h

CMD_WRITE_NA    EQU     080h
CMD_WRITE_ACK   EQU     088h

CMD_WRITE_CONF  EQU     090h
CMD_WRITE_CONF_PERM EQU 098h

CMD_READ        EQU     0A1h
CMD_READ_CONF   EQU     0A9h

CMD_WRITE_BLOCK EQU     0B5h
CMD_READ_BLOCK  EQU     0B9h

; GET_INFO attributes
GET_INFO_GENERAL EQU    0
GET_INFO_WRITE  EQU     1
GET_INFO_READ   EQU     2
GET_INFO_CONF   EQU     3

; Channel attributes
SIZE_8BIT       EQU     1
SIZE_16BIT      EQU     2
SIZE_24BIT      EQU     3
SIZE_32BIT      EQU     4

;--------------------------------------------------------------------
; variables in internal RAM

        DSEG AT 0030h

IN_BUF: 
        DS      10              ; input buffer for serial port

CRC_CODE:
        DS      1               ; running CRC-8 code

STACK:
        DS      30              ; reserve space for stack

RBUF_WP:
        DS      1               ; write and read pointers for receive buffer
RBUF_RP:
        DS      1

;--------------------------------------------------------------------
; bit variables in internal RAM

bit_seg         SEGMENT BIT
                RSEG    bit_seg

DEBUG_MODE:
        DBIT    1               ; debugging mode

LCD_PRESENT:
        DBIT    1               ; flag if LCD display plugged in

TI_SHADOW:
        DBIT    1

;--------------------------------------------------------------------
; program entry locations

        CSEG    AT      0

        LJMP    START


        CSEG    AT      0023h

        LJMP    SERIAL_INT
        
;--------------------------------------------------------------------
; MAIN PROGRAM

        CSEG    AT      004Bh

START:   
        MOV     SP,#STACK       ; initialize stack
        CALL    SETUP           ; setup RS232 etc.

        CLR     TI
        CLR     TI_SHADOW
        CLR     RS485_EN

        CLR     LPT_STATUS      ; status ok

        CLR     LPT_BUSY        ; ready to receive LPT data

        ;SETB    RS485_EN
MAIN_LOOP:
        CALL    WATCHDOG_REFRESH
        CALL    CHECK_LPT
        CALL    CHECK_RBUF

        ;CALL    RS485_TEST
        ;CALL    DELAY_100M

        SJMP    MAIN_LOOP      ; repeat indefinately

;--------------------------------------------------------------------
; Subroutines                                                      
;--------------------------------------------------------------------

;--------------------------------------------------------------------
; Main setup routine

STR_HELLO:
        DB      'SCS-SM1 Ready',0

SETUP:

; clear all dynamic memory

        CLR     DEBUG_MODE
        MOV     RBUF_WP,#0
        MOV     RBUF_RP,#0

; assign and enable port pins

	MOV 	PRT0MX,#00000001b 	; Enable RX/TX

        ; Port configuration (1 = Push Pull Output)
        MOV     PRT0CF,#00000001b       ; P0 0:TX 1:RX 5,6:LED 7:ParPort
        MOV     PRT1CF,#00000000b       ; P1 0-7:PP
        MOV     PRT2CF,#00000000b       ; P2 0:Din 1-7:LCD (ext. PU)
        MOV     PRT3CF,#00000000b       ; P3 5:Enbl 6:Int. 7:LED

        ; Enable watchdog 95ms (@11.0592MHz)
        MOV     WDTCN,#00000111b

        ; Select external quartz oscillator
        MOV     OSCXCN,#01100110b       ; Crystal mode, Power Factor 22E6
        MOV     OSCICN,#00001000b       ; CLKSL=1 (external)

        ; Enable reset pin and watchdog reset
        MOV     RSTSRC,#00001001b

; Set input ports to 1
        MOV     LPT_DATA,#0FFh
        SETB    LPT_NSTROBE
        SETB    LPT_BUSY
        SETB    LPT_NACK
        SETB    LPT_NDATAREADY
        SETB    LPT_BIT9

; Blink LEDs

        MOV     R1,#5
BLINK:
        CLR     LED_GREEN       ; set LEDs
        CLR     LED_YELLOW      ; 
        CLR     LED_RED         ; 

	CALL    DELAY_100M

	SETB    LED_YELLOW      ; clear LEDs
	SETB    LED_RED
        SETB    LED_GREEN

        CALL    DELAY_100M
        DJNZ    R1,BLINK

; setup LCD display

        CALL    LCD_SETUP

; output string to LCD display
        MOV     A,#0
        MOV     B,#0
        CALL    LCD_GOTO

        MOV     DPTR,#STR_HELLO
        CALL    LCD_PUTS

; configure UART

        CALL    UART_INIT
        MOV     A,#3            ; set baud rate
        CALL    SET_BAUD

        SETB    ES              ; enable serial interrupt
        CLR     PS              ; serial interrupt low priority
        SETB    EA              ; general interrupt enable
        CLR     RB8             ; clear read bit9
        
; turn on power LED

        CLR     LED_GREEN

        RET

;--------------------------------------------------------------------
; Test routine to send one byte over RS-485

RS485_TEST:

        SETB    RS485_EN
        CALL    DELAY_10U       ; wait for slow optocoupler

        CLR     LED_YELLOW      ; turn on LED
        ;SETB    TB8
        MOV     A,#55h
        MOV     SBUF,A
        JNB     TI_SHADOW,$
        CLR     TI_SHADOW
        SETB    LED_YELLOW      ; turn off LED

        CALL    DELAY_10U
        CLR     RS485_EN        ; wait for slow optocoupler
        RET

;--------------------------------------------------------------------
; Receive data from LPT

CHECK_LPT:
        JNB     LPT_NSTROBE,NEW_DATA
        RET

NEW_DATA:
        MOV     A,LPT_DATA
        MOV     C,LPT_BIT9      ; get Bit9
        SETB    LPT_BUSY        ; signal busy to PC

        ; send byte to serial port, flash LED
        SETB    RS485_EN
        CALL    DELAY_10U       ; wait for slow optocoupler

        CLR     LED_YELLOW
        MOV     TB8,C
        MOV     SBUF,A
        JNB     TI_SHADOW,$
        CLR     TI_SHADOW
        SETB    LED_YELLOW

        CALL    DELAY_10U       ; wait for slow optocoupler
        CLR     RS485_EN

        ; wait for strobe to be removed
WAIT_NSTROBE:
        CALL    WATCHDOG_REFRESH
        JNB     LPT_NSTROBE,WAIT_NSTROBE

        ; remove busy
        CLR     LPT_BUSY

        RET

;--------------------------------------------------------------------
; Forward data to PC from receive buffer

CHECK_RBUF:
        MOV     A,RBUF_WP
        CJNE    A,RBUF_RP,NEW_RDATA
        RET

NEW_RDATA:
        ; abort if PC strobe is active to avoid data collision
        JB      LPT_NSTROBE,NO_STROBE
        RET

NO_STROBE:
        MOV     R0,RBUF_RP      ; output new data
        MOVX    A,@R0
        INC     RBUF_RP         ; increment read pointer

        MOV     LPT_DATA,A
        CLR     LPT_NDATAREADY  ; signal new data

        ; wait for PC acknowledge        
WAIT_NACK:
        CALL    WATCHDOG_REFRESH
        JB      LPT_NACK,WAIT_NACK

        SETB    LPT_NDATAREADY  ; remove data ready signal
        MOV     LPT_DATA,#0FFh  ; prepare port for input

        ; wait for PC acknowlege to be removed
WAIT_NACK2:
        CALL    WATCHDOG_REFRESH
        JNB     LPT_NACK,WAIT_NACK2

        RET

;--------------------------------------------------------------------
; Serial interrupt

SERIAL_INT:
        PUSH    PSW
        JB      RI,SER_READ

        SETB    TI_SHADOW       ; move TI to TI_SHADOW
        CLR     TI
        POP     PSW
        RETI

SER_READ:
        PUSH    ACC
        PUSH    DPL
        PUSH    DPH
        
        CLR     RS1             ; Select register bank 1
        SETB    RS0
        
        MOV     A,RBUF_WP       ; Check for RBuffer overflow
        INC     A
        CJNE    A,RBUF_RP,NO_OVER
        CLR     RI
        SJMP    SER_RET

NO_OVER:
        MOV     R0,RBUF_WP
        MOV     A,SBUF          ; read character
        CLR     RI              ; clear read flag
        MOVX    @R0,A           ; store character
        INC     R0
        MOV     RBUF_WP,R0

SER_RET:        
        POP     DPH
        POP     DPL
        POP     ACC
        POP     PSW             ; selects old register bank
        RETI


;--------------------------------------------------------------------
;
; CRC-8 code calculation, adds value in A to running CRC code 
; in CRC_CODE
;
; Modifies: DPTR,CRC_CODE
;
;--------------------------------------------------------------------

CRC8_DATA:       
        db 000h, 05eh, 0bch, 0e2h, 061h, 03fh, 0ddh, 083h
        db 0c2h, 09ch, 07eh, 020h, 0a3h, 0fdh, 01fh, 041h
        db 09dh, 0c3h, 021h, 07fh, 0fch, 0a2h, 040h, 01eh
        db 05fh, 001h, 0e3h, 0bdh, 03eh, 060h, 082h, 0dch
        db 023h, 07dh, 09fh, 0c1h, 042h, 01ch, 0feh, 0a0h
        db 0e1h, 0bfh, 05dh, 003h, 080h, 0deh, 03ch, 062h
        db 0beh, 0e0h, 002h, 05ch, 0dfh, 081h, 063h, 03dh
        db 07ch, 022h, 0c0h, 09eh, 01dh, 043h, 0a1h, 0ffh
        db 046h, 018h, 0fah, 0a4h, 027h, 079h, 09bh, 0c5h
        db 084h, 0dah, 038h, 066h, 0e5h, 0bbh, 059h, 007h
        db 0dbh, 085h, 067h, 039h, 0bah, 0e4h, 006h, 058h
        db 019h, 047h, 0a5h, 0fbh, 078h, 026h, 0c4h, 09ah
        db 065h, 03bh, 0d9h, 087h, 004h, 05ah, 0b8h, 0e6h
        db 0a7h, 0f9h, 01bh, 045h, 0c6h, 098h, 07ah, 024h
        db 0f8h, 0a6h, 044h, 01ah, 099h, 0c7h, 025h, 07bh
        db 03ah, 064h, 086h, 0d8h, 05bh, 005h, 0e7h, 0b9h
        db 08ch, 0d2h, 030h, 06eh, 0edh, 0b3h, 051h, 00fh
        db 04eh, 010h, 0f2h, 0ach, 02fh, 071h, 093h, 0cdh
        db 011h, 04fh, 0adh, 0f3h, 070h, 02eh, 0cch, 092h
        db 0d3h, 08dh, 06fh, 031h, 0b2h, 0ech, 00eh, 050h
        db 0afh, 0f1h, 013h, 04dh, 0ceh, 090h, 072h, 02ch
        db 06dh, 033h, 0d1h, 08fh, 00ch, 052h, 0b0h, 0eeh
        db 032h, 06ch, 08eh, 0d0h, 053h, 00dh, 0efh, 0b1h
        db 0f0h, 0aeh, 04ch, 012h, 091h, 0cfh, 02dh, 073h
        db 0cah, 094h, 076h, 028h, 0abh, 0f5h, 017h, 049h
        db 008h, 056h, 0b4h, 0eah, 069h, 037h, 0d5h, 08bh
        db 057h, 009h, 0ebh, 0b5h, 036h, 068h, 08ah, 0d4h
        db 095h, 0cbh, 029h, 077h, 0f4h, 0aah, 048h, 016h
        db 0e9h, 0b7h, 055h, 00bh, 088h, 0d6h, 034h, 06ah
        db 02bh, 075h, 097h, 0c9h, 04ah, 014h, 0f6h, 0a8h
        db 074h, 02ah, 0c8h, 096h, 015h, 04bh, 0a9h, 0f7h
        db 0b6h, 0e8h, 00ah, 054h, 0d7h, 089h, 06bh, 035h

CRC8_ADD:
        PUSH    ACC

        MOV     DPTR,#CRC8_DATA         ; Point To Table
        XRL     A,CRC_CODE              ; XOR In CRC
        MOVC    A,@A+DPTR               ; Get New CRC Byte
        MOV     CRC_CODE,A              ; Store Back

        POP     ACC
        RET

CRC8_ADD1:
        PUSH    ACC
        PUSH    DPL
        PUSH    DPH

        MOV     DPTR,#CRC8_DATA         ; Point To Table
        XRL     A,CRC_CODE              ; XOR In CRC
        MOVC    A,@A+DPTR               ; Get New CRC Byte
        MOV     CRC_CODE,A              ; Store Back

        POP     DPH
        POP     DPL
        POP     ACC
        RET

;--------------------------------------------------------------------
;
; Configure UART via timer 2 
;
; parameters:
;   A=1:9600,2:19200,3:28800,4:57600,5:115200,6:172800,7:345600 Baud
;
; Modifes: A,DPTR
;
;--------------------------------------------------------------------

BAUD_TABLE:     ; reload 100-x, x = 11.0592MHz/32/baud
        DB      100h-36,100h-18,100h-12,100h-6,100h-3,100h-2,100h-1

UART_INIT:
        MOV     SCON,#11010000b ; Mode 3, 9 bit, receive enable
;        MOV     SCON,#50H       ; Mode 1, 8 bit, receive enable

        MOV     TMOD,#00H       ; timer 0&1 no function
        MOV     T2CON,#00110100b; timer 2 RX+TX mode
        MOV     RCAP2H,#0FFh
        MOV     RCAP2L,#100h-36 ; 9600 baud by default
        SETB    TR2

        RET

SET_BAUD:
        CLR     TR2
        MOV     RCAP2H,#0FFh

        MOV     DPTR,#BAUD_TABLE
        DEC     A
        MOVC    A,@A+DPTR
        MOV     RCAP2L,A        ; set reload register from table
        SETB    TR2

        RET

;--------------------------------------------------------------------
;
; Refresh watchdog
;
; Modifes:
;
;--------------------------------------------------------------------

WATCHDOG_REFRESH:

        MOV     WDTCN,#0A5h     ; refresh watchdog timer..
        RET        

;--------------------------------------------------------------------
;
; Delay functions for 11.0520 MHz quartz
;
; Modifies: R6,R7,(R5)
;
;--------------------------------------------------------------------

        ; 11059200Hz -> 1 cycle = 0.09 us

DELAY_1:                        ; delay 1s
        MOV     R5,#100
DLY0:   CALL    DELAY_10M
        CALL    WATCHDOG_REFRESH
        DJNZ    R5,DLY0
        RET


DELAY_100M:                     ; delay 100ms
        MOV     R5,#10
DLY1:   CALL    DELAY_10M
        CALL    WATCHDOG_REFRESH
        DJNZ    R5,DLY1
        RET

DELAY_10M:                      ; delay 1ms
        MOV     R7,#200         ; 200 * 50us = 10ms
DLY2:   MOV     R6,#183         ; 0.18us
        DJNZ    R6,$            ; 183 * 0.27us = 49.41us
        DJNZ    R7,DLY2
        RET                     ; 9 * 0.09us = 0.81us (call+ret)

DELAY_1M:                       ; delay 1ms
        MOV     R7,#20          ; 20 * 50us = 1ms
DLY3:   MOV     R6,#183         ; 0.18us
        DJNZ    R6,$            ; 183 * 0.27us = 49.41us
        DJNZ    R7,DLY3
        RET                     ; 9 * 0.09us = 0.81us (call+ret)

DELAY_100U:                     ; delay 100us
        MOV     R7,#2           ; 2 * 50us
DLY4:   MOV     R6,#181         ; 0.18us
        DJNZ    R6,$            ; 181 * 0.27us = 48.87us
        DJNZ    R7,DLY4
        RET                     ; 9 * 0.09us = 0.81us (call+ret)

DELAY_10U:                      ; delay 10us
        MOV     R6,#33          ; 0.18us
        DJNZ    R6,$            ; 33 * 0.27us = 8.91us
        RET                     ; 9 * 0.09us = 0.81us (call+ret)

DELAY_1U:                       ; delay 1us
        RET                     ; 9 * 0.09us = 0.81us (call+ret)

;--------------------------------------------------------------------
;
; EEPROM functions for reading writing flash EEPROM
;
; Parameters: 
;   A   parameter index 0..N
;   R0  pointer to data
;   R1  data length (1..4)
;
; Modifies:
;   A,R0,R1,R2
;
;--------------------------------------------------------------------

EEPROM_WRITE:
        ; copy page to RAM at 128
        PUSH    DPL
        PUSH    DPH
        PUSH    B
        PUSH    ACC
        MOV     B,R0
        MOV     R0,#80h
        MOV     DPTR,#7F80h
        MOV     R1,#80h
EEW_LOOP1:
        MOV     A,R1
        MOVC    A,@A+DPTR
        MOV     @R1,A
        INC     R1
        DJNZ    R0,EEW_LOOP1

        ; copy data into RAM page
        POP     ACC
        MOV     R0,B

        RL      A
        RL      A
        ADD     A,#80h
        MOV     R1,A
        MOV     R2,#4
EEW_LOOP2:
        MOV     A,@R0
        MOV     @R1,A
        INC     R1
        INC     R0
        DJNZ    R2,EEW_LOOP2

        ; erase page
        MOV     A,FLSCL
        ANL     A,#0F0h
        ORL     A,#00001000b    ; set timer for 11.052 MHz clock
        MOV     FLSCL,A
        MOV     PSCTL,#3        ; allow write+erase
        MOV     DPTR,#8000h
        MOVX    @DPTR,A

        ; write page
        MOV     PSCTL,#1        ; allow write

        MOV     R0,#80h
        MOV     DPTR,#8000h
        MOV     R1,#80h
EEW_LOOP3:
        MOV     A,@R1
        MOVX    @DPTR,A
        INC     R1
        INC     DPL
        DJNZ    R0,EEW_LOOP3

        MOV     PSCTL,#0        ; don't allow write+erase

        POP     B
        POP     DPH
        POP     DPL
        RET

EEPROM_READ:
        MOV     DPTR,#8000h     ; read from 128-byte EEPROM page
        RL      A
        RL      A
EER_LOOP:
        PUSH    ACC
        MOVC    A,@A+DPTR
        MOV     @R0,A
        POP     ACC
        INC     R0
        INC     A
        DJNZ    R1,EER_LOOP
        RET

;--------------------------------------------------------------------
;
; LCD functions for HD44780/KS0066 compatible LCD displays
;
; Modifes: R6,R7 (via DELAY), A, C, DPTR
;
;--------------------------------------------------------------------

LCD     EQU     P2              ; LCD display connected to port2

LCD_RS  EQU     LCD.1
LCD_R_W EQU     LCD.2
LCD_E   EQU     LCD.3

LCD_DB4 EQU     LCD.4
LCD_DB5 EQU     LCD.5
LCD_DB6 EQU     LCD.6
LCD_DB7 EQU     LCD.7

LCD_CLK EQU     LCD.7           ; pins for 4021 shift register
LCD_P_W EQU     LCD.6
LCD_SD  EQU     LCD.0

LCD_SETUP:
        MOV     LCD,#0
        CALL    DELAY_100M
        MOV     LCD,#00110000b
        SETB    LCD_E
        CALL    DELAY_1U
        CLR     LCD_E
        CALL    DELAY_100M
        SETB    LCD_E
        CALL    DELAY_1U
        CLR     LCD_E
        CALL    DELAY_100M
        SETB    LCD_E
        CALL    DELAY_1U
        CLR     LCD_E
        CALL    DELAY_100M

        MOV     LCD,#00100000b  ; set 4-bit interface
        SETB    LCD_E
        CALL    DELAY_1U
        CLR     LCD_E
        CALL    DELAY_100M

        ; test if LCD present
        MOV     LCD,#11111100b  ; data input R/W=H
        CALL    DELAY_10U
        MOV     A,LCD
        JNB     ACC.7,LCD_OK

        CLR     LCD_PRESENT
        RET

LCD_OK:
        SETB    LCD_PRESENT

        CLR     C               ; indicate control
        MOV     A,#00101100b    ; select 2 lines, big font
        CALL    LCD_OUT

        MOV     A,#00001100b    ; display on
        CALL    LCD_OUT

        MOV     A,#00000001b    ; clear display
        CALL    LCD_OUT

        MOV     A,#00000110b    ; set entry mode
        CALL    LCD_OUT

        RET

LCD_CLEAR:                      ; clear display
        PUSH    ACC
        CLR     C               ; indicate command
        MOV     A,#1
        CALL    LCD_OUT
        POP     ACC
        RET

LCD_GOTO:                       ; goto position A(0..19),B(0..1)
        PUSH    PSW
        PUSH    ACC
        MOV     C,B.0
        MOV     ACC.6,C
        SETB    ACC.7
        CLR     C
        CALL    LCD_OUT
        POP     ACC
        POP     PSW
        RET

LCD_PUTS:                       ; output zero terminated string
        PUSH    PSW
        PUSH    ACC

        SETB    C               ; indicate data
        CLR     A
        MOV     R0,A
PUTS_LOOP:
        MOV     A,R0
        MOVC    A,@A+DPTR
        JZ      PUTS_RET
        CALL    LCD_OUT
        INC     R0
        SJMP    PUTS_LOOP
PUTS_RET:
        CLR     C

        POP     ACC
        POP     PSW
        RET

LCD_HEX:                        ; output ACC in hex format
        PUSH    ACC
        PUSH    ACC
        SWAP    A
        ANL     A,#00001111b
        CALL    LCD_HEX_NIBBLE
        POP     ACC
        ANL     A,#00001111b
        CALL    LCD_HEX_NIBBLE
        POP     ACC
        RET

LCD_HEX_NIBBLE:
        CLR     C
        SUBB    A,#10
        JNC     HEX_A_F
        ADD     A,#10
        ADD     A,#'0'
        SETB    C
        CALL    LCD_OUT
        RET
HEX_A_F:
        ADD     A,#'A'
        SETB    C
        CALL    LCD_OUT
        RET       

LCD_PUTC:
        SETB    C

LCD_OUT:                        ; move ACC to LCD via 4-bit interface
        JB      LCD_PRESENT, LCD_OUT1
        RET

LCD_OUT1:
        PUSH    ACC
        PUSH    B
        MOV     B,A
        MOV     LCD,#11111100b  ; data input R/W=H
        CALL    DELAY_10U
        CALL    DELAY_10U
        CALL    DELAY_10U
LCD_BUSY:
        CALL    WATCHDOG_REFRESH
        MOV     A,LCD
        JB      ACC.7,LCD_BUSY  ; loop if BF
        MOV     LCD,#0
        CALL    DELAY_1U

        MOV     A,B
        ANL     A,#11110000b
        MOV     ACC.1,C         ; set RS according to C
        MOV     LCD,A
        CALL    DELAY_10U       ; let signals settle
        SETB    LCD_E
        CALL    DELAY_10U
        CLR     LCD_E
        MOV     A,B
        SWAP    A
        ANL     A,#11110000b
        MOV     ACC.1,C
        MOV     LCD,A
        CALL    DELAY_10U       ; let signals settle
        SETB    LCD_E
        CALL    DELAY_10U
        CLR     LCD_E
        POP     B
        POP     ACC
        RET

;--------------------------------------------------------------------
;
; Input function for SCS-LCD1
;
; Modifes: R0,C, R6,R7 (via delay)
; Returns: switch states in ACC
;
;--------------------------------------------------------------------

SCS_LCD1_READ:
        CLR     LCD_CLK
        SETB    LCD_SD          ; enable input
        
        SETB    LCD_P_W         ; latch input
        CALL    DELAY_1U
        CLR     LCD_P_W
        
        MOV     C,LCD_SD        ; get first bit
        RRC     A               ; rol carry bit in A

        MOV     R0,#7
SCS_LCD1_LOOP:

        SETB    LCD_CLK
        CALL    DELAY_1U
        CLR     LCD_CLK
        CALL    DELAY_1U
        
        MOV     C,LCD_SD
        RRC     A               ; rol carry bit in A

        DJNZ    R0,SCS_LCD1_LOOP
        
        RET

;--------------------------------------------------------------------

END
